﻿using StyleCop;
using StyleCop.CSharp;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace CLK.StyleCop.Rules
{
    [SourceAnalyzer(typeof(CsParser))]
    public class FieldNamesMustBeginWithUnderscoreRule : SourceAnalyzer
    {
        // Methods
        public override void AnalyzeDocument(CodeDocument document)
        {
            // Document
            var csDocument = document as CsDocument;
            if (csDocument == null) return;

            // Element
            var element = csDocument.RootElement;
            if (element == null) return;

            // Analyze
            if (element.Generated == false)
            {
                csDocument.WalkDocument(new CodeWalkerElementVisitor<object>(this.AnalyzeElement), null, null);
            }
        }

        private bool AnalyzeElement(CsElement element, CsElement parentElement, object context)
        {
            // Analyze
            if (element.Generated == false && element.ElementType == ElementType.Field && element.ActualAccess == AccessModifierType.Private)
            {
                // Field
                var field = element as Field;
                if (field == null) return true;

                // Unique
                if (field.Const == true) return true;
                if (field.Static == true && field.Readonly == true) return true;

                // Normal
                if (field.Declaration.Name.ToCharArray()[0] != '_')
                {
                    this.AddViolation(field, "FieldNamesMustBeginWithUnderscoreRule");
                }
            }

            // Return
            return true;
        }
    }
}